#!/usr/bin/perl -I/usr/local/sauron -w
#
#
#=================================
# project: Sauron DHCP import
# script:  add_host_options.pl
# version: 1.0\n";
#
# Automaticke pridani "options"
# zaznamum u jednotlivych hostu.
# (Poustet az po importu dat do
# Saurona - vyuziva shody HW-ADR)
#=================================


#------------------------
# zacatek Sauroniho kodu
#------------------------

require 5;
use Getopt::Long;
use Sauron::DB;
use Sauron::Util;
use Sauron::BackEnd;
use Sauron::Sauron;

load_config();

fatal("cannot find $PROG_DIR/runsql (invalid configuration?)")
unless (-x "$PROG_DIR/runsql");

db_connect();
  
#----------------------
# konec Sauroniho kodu
#----------------------

mkdir TEMP;

print "Acquiring ID list (enter Sauron database password) ...\n";
`psql -c "select id, ether from hosts" sauron sauron > TEMP/id1.txt`;

print "Acquiring group ID (enter Sauron database password) ...\n";
`psql -c "select id, name from groups" sauron sauron > TEMP/id2.txt`;

#nastaveni jmen souboru
open(ID, "<TEMP/id1.txt");
open(ID2, "<TEMP/id2.txt");
open(DHCP, "<DHCP-COMPLET/import2.dhcp");

#---------------------------------------------------------------------------
# Priprava pole obsahujiho usporadanou dvojici ID-HW
#---------------------------------------------------------------------------

#nacteni dat
@iddata = <ID>;
if ($#iddata == 0){
  print "Error: no ID detected.\n";
  exit(1);
}

#lepsi naformatovani ID=HW
@tempdata = ();
foreach(@iddata){
  if ((($_ =~ /[0-9]/) == 1) && (($_ =~ /.*rows.*/) == 0)){
    #vynechani hlavicky
    $_ =~ /[0-9][0-9]*/;
    $part1 = $&;
    $_ =~ /\|.*/;
    $part2 = $&;
    $part2 =~ /[0-9A-F][0-9A-F]*/;
    $part2 = $&;
    if ($part2 =~ /[0-9A-F]/){
      #tj. pouze zaznamy u nichz je hw adresa definovana
      push @tempdata, "$part1=$part2";
    }
  }
}
@iddata = @tempdata;

close(ID);

#---------------------------------------------------------------------------
# Nalezeni ID pro pozadovanou skupinu
#---------------------------------------------------------------------------

#nacteni dat
@groupdata = <ID2>;
if ($#groupdata == 0){
  print "Error: no ID detected.\n";
  exit(1);
}

print "Enter group name: ";
$name = <STDIN>;
$name =~ "[a-zA-Z0-9-_]*";
$name = $&;

$groupid = -1;

#lepsi naformatovani ID=name
@tempdata = ();
foreach(@groupdata){
  if ((($_ =~ /[0-9]/) == 1) && (($_ =~ /.*rows.*/) == 0)){
    #vynechani hlavicky
    $_ =~ /[0-9][0-9]*/;
    $part1 = $&;
    $_ =~ /\|.*/;
    $part2 = $&;
    $part2 =~ /[0-9A-Za-z-_][0-9A-Za-z-_]*/;
    $part2 = $&;
    #printf "'$part2'\n";
    if ($part2 eq $name){
      #tj. zaznam pro hledanou skupinu
      $groupid = $part1;
    }
  }
}

if ($groupid == -1){
  print "Error: group not found.\n";
   exit(1);
}
else{
  print "Group ID = $groupid\n";
}

close(ID2);


#---------------------------------------------------------------------------
# Priprava pole obsahujiho usporadanou dvojici HW-option
#---------------------------------------------------------------------------

print "Searching for hosts in file DHCP-cOMPLET/import2.dhcp ... \n";

#nacteni dat
@dhcp = <DHCP>;

$open_host = 0;
$host_name = "---";
$temp_hw = "0";
$fixed_detected = 0;
@temp_options = ();
@sql_lines = ();
foreach(@dhcp){
  $process_line = 1;
  if ($_ =~ /^\s*host.*/){
    $_ =~ /^\s*host\s*/;
    $tempname = $';
    #print "temp = '$tempname'\n";
    $tempname =~ /\s*{\s*.*/;
    $host_name = $`;
    #print "HN = '$host_name'\n";
    #nalezeni zacatku hosta
    $open_host = 1;
    $process_line = 0;
    $fixed_detected = 0;
  }
  
  if (($_ =~ /^\s*}.*/) && ($open_host == 1)){
    #nalezeni konce hosta    
    if ($temp_hw eq "0"){
      print "Missed hw address.\n";
    }
    else{
      if ($fixed_detected == 0){
        #jedna se o netobook
        modify_hw($temp_hw);
        $id = &get_id($temp_hw);
        if ($id == -1){
          printf "No ID for hw adress = '$temp_hw' ('$host_name').. skipping\n";
          #printf "$host_name\n";
        }
        else{
#          foreach(@temp_options){
#            $_ =~ /[^\s].*[^;\n]/;
            push @sql_lines, "insert into group_entries (host, grp) values ($id, $groupid);\n";
#          }
          #print "$temp_hw\n";
        }
      }
    }
    $open_host = 0;
    $host_name = "---";
    @temp_options = ();
    $temp_hw = "0";
    $id = -1;
  }

  if (($process_line == 1) && ($open_host == 1)){
    if ($_ =~ /^\s*hardware ethernet.*/){
      $_ =~ /[0-9A-Fa-f]{2}:[0-9A-Fa-f]{2}:[0-9A-Fa-f]{2}:[0-9A-Fa-f]{2}:[0-9A-Fa-f]{2}:[0-9A-Fa-f]{2}/;
      $temp_hw = $&;
      #print "$temp_hw\n";
    }
    else{
      if ($_ =~/^\s*fixed-address.*/){
       #vynechat - uz je importovano sauronem
       $fixed_detected = 1;
      }
      else{
        #radek s option, ktery Sauron neumi importovat
	#push @temp_options, $_;
      }
    } #hw-ethernet
  }
}

close(DHCP);

#---------------------------------------------------------------------------
# Priprava SQL prikazu pro pridani pozadovanych zaznamu
#---------------------------------------------------------------------------
print "Generating SQL commands ...\n";
open(SQL, ">TEMP/insert.sql");
foreach(@sql_lines){
  print SQL $_;
}
close(SQL);

print "Executing SQL commands ...\n";

$res=system("$PROG_DIR/runsql","TEMP/insert.sql");
fatal("failed") if ($res);

print "Host options were successfully inserted.\n\n";
#---------------------------------------------------------------------------
# Podprogramy
#---------------------------------------------------------------------------
sub get_id{
  $hwsearch=$_[0];
  foreach(@iddata){
    $_ =~ /=/;
    $id = $`;
    $hw = $';
    if ($hw eq $hwsearch){
      ##print "$hw : $id\n";
      #prevod na format jako je v databazi
      return ($id);
    }
  }
  return(-1);
}

sub modify_hw {
   for (@_) { tr/a-z/A-Z/ };
   for (@_) { s/://};
   for (@_) { s/://};
   for (@_) { s/://};
   for (@_) { s/://};
   for (@_) { s/://};
}

